@@ -228,13 +228,16 @@ var92 = {logical(mod(reshape(0:19,4,5),2));~mod(reshape(0:19,4,5),2);...
 
 var93 = cell(2,3);
 var94 = cell2struct(var93, {'field1', 'field2'}, 1);
+% slCharacterEncoding('UTF-8');
+global var95;
+var95 = char([1576, 1580, 1604, 1740; 273, 105, 7879, 110]);
 
 save('-v6',['matio_test_cases_uncompressed' e_str '.mat'],'var*');
 save('-v7',['matio_test_cases_compressed' e_str '.mat'],'var*');
 save('-v7.3',['matio_test_cases_compressed_hdf' e_str '.mat'],'var*');
 save('-v7.3','-nocompression',['matio_test_cases_uncompressed_hdf' e_str '.mat'],'var*');
 save('-v4',['matio_test_cases_v4' e_str '.mat'],'var1','var11','var21',...
-     'var22','var24');
+     'var22','var24','var95');
 
 x = pi;
 save('-v4',['small_v4' e_str '.mat'],'x');
